/*
 * Copyright 2004-2014 ICEsoft Technologies Canada Corp.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package org.icepush;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Group {
    private static final Logger LOGGER = Logger.getLogger(Group.class.getName());

    private final LocalPushGroupManager localPushGroupManager;
    private final String name;
    private final Set<String> pushIdList = new HashSet<String>();

    protected Group(final String name, final String firstPushId, final LocalPushGroupManager localPushGroupManager) {
        this.name = name;
        this.localPushGroupManager = localPushGroupManager;
        addPushID(firstPushId);
        if (LOGGER.isLoggable(Level.FINE)) {
            LOGGER.log(Level.FINE, "Push Group '" + this.name + "' created.");
        }
    }

    public String getName() {
        return name;
    }

    void addPushID(final String pushId) {
        pushIdList.add(pushId);
    }

    String[] getPushIDs() {
        return pushIdList.toArray(new String[pushIdList.size()]);
    }

    public void removePushID(final String pushId) {
        pushIdList.remove(pushId);
        if (pushIdList.isEmpty()) {
            if (LOGGER.isLoggable(Level.FINE)) {
                LOGGER.log(
                    Level.FINE, "Disposed Push Group '" + name + "' since it no longer contains any PushIDs.");
            }
            localPushGroupManager.removeGroup(name);
        }
    }
}
